import { Object3D, TextureLoader, Vector3 } from "three";
import useModelStore from "@/store/useModelStore.ts";
import { useState } from "react";
import { models } from "@/constant/models.ts";
import { textures } from "@/constant/textures.ts";

const CapMesh = ({
  areaIndex,
  mesh,
}: {
  areaIndex: number;
  mesh: Object3D;
}) => {
  const logo = useTexture("/textures/archlogo.png");
  const activeModel = useModelStore((state) => state.activeModel);
  const activeTextures = useModelStore((state) => state.activeTextures);
  const [pos, setPos] = useState<Vector3>(new Vector3(0, 0, 0));
  return (
    <mesh
      onClick={(ev) => {
        const { x, y, z } = ev.point;
        setPos(
          new Vector3(
            x / models[activeModel].scale,
            y / models[activeModel].scale,
            z / models[activeModel].scale,
          ),
        );
      }}
      castShadow
      receiveShadow
      // eslint-disable-next-line @typescript-eslint/ban-ts-comment
      // @ts-expect-error
      geometry={mesh.geometry}
      dispose={null}
    >
      {areaIndex > -1 ? (
        <meshStandardMaterial
          map={new TextureLoader().load(
            textures[activeTextures[areaIndex]].path,
          )}
        />
      ) : (
        <meshStandardMaterial color="gray" />
      )}
      <Decal position={pos} scale={0.05} rotation={Math.PI}>
        <meshPhysicalMaterial
          transparent
          polygonOffset
          polygonOffsetFactor={-10}
          map={logo}
          map-flipY={false}
          map-anisotropy={16}
          iridescence={1}
          iridescenceIOR={1}
          iridescenceThicknessRange={[0, 1400]}
          roughness={1}
          clearcoat={0.5}
          metalness={0.75}
          toneMapped={false}
        />
      </Decal>
    </mesh>
  );
};
