import { Tab, TabList, TabPanel, TabPanels, Tabs } from "@chakra-ui/react";
import { models } from "@/constant/models.ts";
import useModelStore from "@/store/useModelStore.ts";
import TextureSelectorPanel from "./TextureSelectorPanel";
import LogoPanel from "./LogoPanel";
import TextLabelPanel from "./TextLabelPanel";

const RightPanel = () => {
  const activeModel = useModelStore((state) => state.activeModel);
  const setActiveModel = useModelStore((state) => state.setActiveModel);
  return (
    <>
      <Tabs variant={"soft-rounded"}>
        <TabList>
          <Tab>版型</Tab>
          <Tab>纹理</Tab>
          <Tab>文字</Tab>
          <Tab>LOGO</Tab>
        </TabList>
        <TabPanels>
          <TabPanel>
            <div className={"grid w-full grid-cols-6 gap-4"}>
              {models.map((model, index) => (
                <div
                  key={model.path}
                  onClick={() => {
                    setActiveModel(index);
                  }}
                  className={`${
                    activeModel === index ? "border-2 border-green-300" : ""
                  } flex aspect-square items-center justify-center overflow-hidden rounded-xl bg-blue-100`}
                >
                  <img
                    className="h-full w-full object-cover"
                    src={models[activeModel].icon}
                  />
                </div>
              ))}
            </div>
          </TabPanel>
          <TabPanel>
            <TextureSelectorPanel />
          </TabPanel>
          <TabPanel>
            <TextLabelPanel />
          </TabPanel>
          <TabPanel>
            <LogoPanel />
          </TabPanel>
        </TabPanels>
      </Tabs>
    </>
  );
};

export default RightPanel;
