import useModelStore, { StickerType } from "@/store/useModelStore";
import { AddIcon, DeleteIcon, EditIcon } from "@chakra-ui/icons";
import {
  Button,
  FormControl,
  FormLabel,
  IconButton,
  Input,
} from "@chakra-ui/react";
import { useState } from "react";
import { fabric } from "fabric";
import { Vector3 } from "three";
import { v4 } from "uuid";

const TextLabelPanel = () => {
  const decals = useModelStore((state) => state.decals);
  const setDecals = useModelStore((state) => state.setDecals);
  const setActiveDecal = useModelStore((state) => state.setActiveDecal);
  const activeDecal = useModelStore((state) => state.activeDecal);
  const [editing, setEditing] = useState(false);
  const [text, setText] = useState("");
  const [color, setColor] = useState("#ffffff");
  return (
    <>
      {editing ? (
        <div className={"rounded bg-white p-4"}>
          <h2 className="font-bold">添加文字</h2>
          <FormControl>
            <FormLabel>文字</FormLabel>
            <Input
              value={text}
              onInput={(ev) => {
                setText(() => (ev.target as HTMLInputElement).value);
              }}
            />
          </FormControl>
          <FormControl>
            <FormLabel>文字颜色</FormLabel>
            <Input
              type="color"
              value={color}
              onInput={(ev) => {
                setColor(() => (ev.target as HTMLInputElement).value);
              }}
            />
          </FormControl>
          <div className={"mt-2 flex justify-center"}>
            <Button
              colorScheme="messenger"
              size={"sm"}
              onClick={() => {
                const canvas = document.createElement("canvas");
                const fabricText = new fabric.Text(text, {
                  fill: color,
                });
                const fabricCanvas = new fabric.Canvas(canvas, {
                  width: fabricText.width,
                  height: fabricText.height,
                });

                fabricCanvas.add(fabricText);
                const id = v4();
                setActiveDecal(id);
                setDecals([
                  ...decals,
                  {
                    id,
                    postion: new Vector3(0, 0, 0),
                    text,
                    url: fabricCanvas.toDataURL(),
                    type: StickerType.text,
                  },
                ]);
                setText(() => "");
                setColor(() => "#ffffff");
                setEditing(() => false);
              }}
            >
              确定
            </Button>
          </div>
        </div>
      ) : (
        <ul>
          {decals
            .filter((el) => el.type === StickerType.text)
            .map((text) => (
              <li
                onClick={() => {
                  setActiveDecal(text.id);
                }}
                key={text.id}
                className={`mt-2 flex h-12 w-full cursor-pointer items-center justify-between rounded bg-white px-2 shadow ${
                  activeDecal === text.id ? "border-2 border-green-100" : ""
                }`}
              >
                <span>{text.text}</span>
                <div className={"right"}>
                  <IconButton
                    onClick={(ev) => {
                      ev.stopPropagation();
                    }}
                    size={"xs"}
                    icon={<EditIcon />}
                    aria-label={""}
                  />
                  <IconButton
                    className={"ml-1"}
                    onClick={(ev) => {
                      ev.stopPropagation();
                    }}
                    size={"xs"}
                    icon={<DeleteIcon />}
                    aria-label={""}
                  />
                </div>
              </li>
            ))}
        </ul>
      )}
      <div className="mt-4 flex justify-center">
        <Button
          colorScheme="messenger"
          onClick={() => {
            // setOpen(() => true)
            setEditing(() => true);
          }}
          leftIcon={<AddIcon />}
        >
          添加文字
        </Button>
      </div>
    </>
  );
};

export default TextLabelPanel;
