import { models } from "@/constant/models";
import { textures } from "@/constant/textures";
import useModelStore from "@/store/useModelStore";

function TextureSelectorPanel() {
  const activeTextures = useModelStore((state) => state.activeTextures);
  const activeArea = useModelStore((state) => state.activeArea);
  const setActiveArea = useModelStore((state) => state.setActiveArea);
  const activeModel = useModelStore((state) => state.activeModel);
  const setActiveTextures = useModelStore((state) => state.setActiveTextures);
  return (
    <>
      <h2 className={"text-sm font-bold"}>选择区域</h2>
      <div className="mt-2 grid grid-cols-8 gap-3">
        {models[activeModel].mesh.map((mesh, index) => (
          <div
            onClick={() => {
              setActiveArea(index);
            }}
            className={`${
              activeArea === index ? "border-2 border-green-300" : ""
            } aspect-square cursor-pointer overflow-hidden rounded`}
            key={mesh.name}
          >
            <img src={mesh.icon} className="h-full w-full object-cover" />
          </div>
        ))}
      </div>
      <h2 className={"mt-3 text-sm font-bold"}>选择纹理</h2>
      <div className="mt-2 grid grid-cols-6 gap-3">
        {textures.map((texture, index) => (
          <div
            key={texture.path}
            onClick={() => {
              setActiveTextures(activeArea, index);
            }}
            className={`${
              activeTextures[activeArea] === index
                ? "border-2 border-green-300"
                : ""
            } cursor-pointer overflow-hidden rounded`}
          >
            <img className={"h-full w-full object-cover"} src={texture.path} />
          </div>
        ))}
      </div>
    </>
  );
}

export default TextureSelectorPanel;
