import { Canvas } from "@react-three/fiber";
import { TextureLoader, Vector3 } from "three";
import { Environment, OrbitControls } from "@react-three/drei";
import useModelStore from "@/store/useModelStore.ts";
import { models } from "@/constant/models.ts";
import { useGLTF } from "@react-three/drei/core/useGLTF";
import { textures } from "@/constant/textures.ts";
import Sticker from "./Sticker";
import { useWhyDidYouUpdate } from "ahooks";

const CapModel = () => {
  const activeModel = useModelStore((state) => state.activeModel);
  const activeTextures = useModelStore((state) => state.activeTextures);
  const decalDragging = useModelStore((state) => state.decalDragging);
  const activeDecal = useModelStore((state) => state.activeDecal);
  const setDecalPositon = useModelStore((state) => state.setDecalPositon);
  useWhyDidYouUpdate('useWhyDidYouUpdateComponent', {
    activeDecal, decalDragging, activeTextures, activeModel
  });
  const { nodes } = useGLTF(models[activeModel].path);
  return (
    <>
      <group
        scale={models[activeModel].scale}
        onPointerMove={(ev) => {
          ev.stopPropagation();
          if (!decalDragging) return;
          if (activeDecal) {
            const { x, y, z } = ev.point;
            const pos = new Vector3(
              x / models[activeModel].scale,
              y / models[activeModel].scale,
              z / models[activeModel].scale,
            );
            setDecalPositon(activeDecal, pos);
          }
        }}
      >
        {Object.keys(nodes).map((keyName) => {
          if (nodes[keyName].type === "Mesh") {
            const areaIndex = models[activeModel].mesh.findIndex(
              (el) => el.name === nodes[keyName].name,
            );
            return (
              <mesh
                key={keyName}
                onDoubleClick={(ev) => {
                  ev.stopPropagation();
                  if (activeDecal) {
                    const { x, y, z } = ev.point;
                    const pos = new Vector3(
                      x / models[activeModel].scale,
                      y / models[activeModel].scale,
                      z / models[activeModel].scale,
                    );
                    console.log(pos);
                    setDecalPositon(activeDecal, pos);
                  }
                }}
                castShadow
                receiveShadow
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-expect-error
                geometry={nodes[keyName].geometry}
                dispose={null}
              >
                {areaIndex > -1 ? (
                  <>
                    <meshStandardMaterial
                      map={new TextureLoader().load(
                        textures[activeTextures[areaIndex]].path,
                      )}
                    />
                    <Stickers />
                  </>
                ) : (
                  <meshStandardMaterial color="gray" />
                )}

              </mesh>
            );
          }
        })}
      </group>
      <OrbitControls enabled={!decalDragging} />
    </>
  );
};



export const Stickers = () => {
  const decals = useModelStore((state) => state.decals);

  return (
    <>
      {decals.map((decal) => (
        <Sticker decal={decal} key={decal.id} />
      ))}
    </>
  )
}



const ThreeScene = () => {
  return (
    <Canvas>
      <ambientLight></ambientLight>
      <pointLight position={[10, 10, 10]}></pointLight>
      <perspectiveCamera />
      <CapModel />
      <Environment preset={"city"} background={false} />
    </Canvas>
  );
};

export default ThreeScene;
